/************************************************************************/
/* Program Name: PM-_lib.h                                              */
/* =======================                                              */
/* BASED ON THE NXC LIBRARY WRITTEN BY DEEPAK PATIL                     */
/* Similar interface for easy migration with some extensions            */
/*                                                                      */
/* This program is free software. You can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; version 3 of the License.              */
/* Read the license at: http://www.gnu.org/licenses/gpl.txt             */
/*                                                                      */
/* When          Who                   Comments                         */
/* 18 DEC 2009   Rudolf Lapie          Initial authoring                */
/************************************************************************/
#pragma systemFile

#ifndef __PM_LIB_INCLUDED__
#define __PM_LIB_INCLUDED__

////////////////////////////////////////////////////////////////////////////////
// INTERFACE ROUTINES
////////////////////////////////////////////////////////////////////////////////
void PM_GetFirmware (tSensors port, byte i2cAddr, string &result) ;
void PM_GetVendorID (tSensors port, byte i2cAddr, string &result) ;
void PM_GetDeviceID (tSensors port, byte i2cAddr, string &result) ;
int PM_ResetCounters(tSensors port, byte i2cAddr) ;      // Reset all sensor registers
int PM_ReadPresentCurrent(tSensors port, byte i2cAddr) ; // Present Current in mA
int PM_ReadPresentVoltage(tSensors port, byte i2cAddr) ; // Present Voltage in mV
long PM_ReadElapsedTime(tSensors port, byte i2cAddr) ;   // Time elapsed since last reset command
// Routines below are already implemented, but not available yet in the PowerMeter
int PM_ReadCapacityUsed(tSensors port, byte i2cAddr) ;   // Capacity used since last reset command
int PM_ReadPresentPower (tSensors port, byte i2cAddr) ; // Present power
long PM_TotalPowerConsumed (tSensors port, byte i2cAddr) ; // Total power consumed since last reset
int PM_ReadMinCurrent(tSensors port, byte i2cAddr) ; // Min Current in mA since last reset
int PM_ReadMaxCurrent(tSensors port, byte i2cAddr) ; // Max Current in mA since last reset
int PM_ReadMinVoltage(tSensors port, byte i2cAddr) ; // Min Voltage in mV since last reset
int PM_ReadMaxVoltage(tSensors port, byte i2cAddr) ; // Max Voltage in mV since last reset
long PM_ReadUserGain (tSensors port, byte i2cAddr) ;
byte PM_ReadGain (tSensors port, byte i2cAddr) ;

////////////////////////////////////////////////////////////////////////////////
// FORWARD DECLARATIONS OF AUXILIARY ROUTINES (do not use these in main program)
////////////////////////////////////////////////////////////////////////////////
static void _PM_ReadString (tSensors port, byte i2cAddr, byte pm_register, string &result) ;
static int _PM_ReadByte (tSensors port, byte i2cAddr, byte pm_register) ;
static int _PM_ReadInteger (tSensors port, byte i2cAddr, byte pm_register) ;
static long _PM_ReadLong (tSensors port, byte i2cAddr, byte pm_register) ;

////////////////////////////////////////////////////////////////////////////////
// CONSTANTS
////////////////////////////////////////////////////////////////////////////////
#define PM_ADDRESS          0x12
#define PM_FIRMWARE         0x00
#define PM_VENDOR_ID        0x08
#define PM_DEVICE_ID        0x10
#define PM_COMMAND_REGISTER 0x41
#define PM_PRESENT_CURRENT  0x42
#define PM_PRESENT_VOLTAGE  0x44
#define PM_CAPACITY_USED    0x46
#define PM_PRESENT_POWER    0x48
#define PM_TOTAL_POWER      0x4A
#define PM_MAX_CURRENT      0x4E
#define PM_MIN_CURRENT      0x50
#define PM_MAX_VOLTAGE      0x52
#define PM_MIN_VOLTAGE      0x54
#define PM_ELAPSED_TIME     0x56
#define PM_USER_GAIN        0x58
#define PM_GAIN             0x5C

////////////////////////////////////////////////////////////////////////////////
// IMPLEMENTATION OF INTERFACE ROUTINES (for the interested only)
////////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------
void PM_GetFirmware(tSensors port, byte i2cAddr, string &result)
{
   _PM_ReadString(port, i2cAddr, PM_FIRMWARE, result);
}
//---------------------------------------------------------------
void PM_GetVendorID(tSensors port, byte i2cAddr, string &result)
{
  _PM_ReadString(port, i2cAddr, PM_VENDOR_ID, result);
}
//---------------------------------------------------------------
void PM_GetDeviceID(tSensors port, byte i2cAddr, string &result)
{
  _PM_ReadString(port, i2cAddr, PM_DEVICE_ID, result );
}
//---------------------------------------------------------------
int PM_ResetCounters(tSensors port, byte i2cAddr)
{
  byte I2CMessage[4];

  I2CMessage[0] = 3; // Message size
	I2CMessage[1] = i2cAddr; // I2C Device Address
	I2CMessage[2] = PM_COMMAND_REGISTER ; // I2C Register to read
	I2CMessage[3] = 'R' ; // Command to send


	while (nI2CStatus[port] == STAT_COMM_PENDING) wait1Msec(1);
  sendI2CMsg(port, I2CMessage[0], 4);
	while (nI2CStatus[port] == STAT_COMM_PENDING) wait1Msec(1);
  return nI2CStatus[port];
}

//---------------------------------------------------------------
int PM_ReadPresentCurrent(tSensors port, byte i2cAddr)
{
  return ( _PM_ReadInteger(port, i2cAddr, PM_PRESENT_CURRENT));
}

//---------------------------------------------------------------
int PM_ReadPresentVoltage(tSensors port, byte i2cAddr)
{
  return ( _PM_ReadInteger(port, i2cAddr, PM_PRESENT_VOLTAGE));
}

//---------------------------------------------------------------
long PM_ReadElapsedTime(tSensors port, byte i2cAddr)
{
	return ( _PM_ReadLong(port, i2cAddr, PM_ELAPSED_TIME ));
}

//---------------------------------------------------------------
int PM_ReadCapacityUsed(tSensors port, byte i2cAddr)
{
  return ( _PM_ReadInteger(port, i2cAddr, PM_CAPACITY_USED));
}

//---------------------------------------------------------------
int PM_ReadPresentPower (tSensors port, byte i2cAddr)
{
  return ( _PM_ReadInteger(port, i2cAddr, PM_PRESENT_POWER));
}
//---------------------------------------------------------------
long PM_TotalPowerConsumed (tSensors port, byte i2cAddr)
{
  return ( _PM_ReadLong(port, i2cAddr, PM_TOTAL_POWER));
}

//---------------------------------------------------------------
int PM_ReadMinCurrent(tSensors port, byte i2cAddr)
{
  return ( _PM_ReadInteger(port, i2cAddr, PM_MIN_CURRENT));
}

//---------------------------------------------------------------
int PM_ReadMaxCurrent(tSensors port, byte i2cAddr)
{
  return ( _PM_ReadInteger(port, i2cAddr, PM_MAX_CURRENT));
}

//---------------------------------------------------------------
int PM_ReadMinVoltage(tSensors port, byte i2cAddr)
{
  return ( _PM_ReadInteger(port, i2cAddr, PM_MIN_VOLTAGE));
}

//---------------------------------------------------------------
int PM_ReadMaxVoltage(tSensors port, byte i2cAddr)
{
  return ( _PM_ReadInteger(port, i2cAddr, PM_MAX_VOLTAGE));
}

//---------------------------------------------------------------
long PM_ReadUserGain (tSensors port, byte i2cAddr)
{
  return ( _PM_ReadLong(port, i2cAddr, PM_USER_GAIN));
}

//---------------------------------------------------------------
byte PM_ReadGain (tSensors port, byte i2cAddr)
{
  return ( _PM_ReadByte(port, i2cAddr, PM_GAIN));
}

////////////////////////////////////////////////////////////////////////////////
// IMPLEMENTATION OF AUXILIARY ROUTINES (for the interested only)
////////////////////////////////////////////////////////////////////////////////

static int _PM_ReadByte (tSensors port, byte i2cAddr, byte pm_register)
{
  ubyte replyMessage[2] ;
  ubyte I2CMessage[4];

  I2CMessage[0] = 2; // Message size
	I2CMessage[1] = i2cAddr; // I2C Device Address
	I2CMessage[2] = pm_register ; // I2C Register to read

	while (nI2CStatus[port] == STAT_COMM_PENDING) wait1Msec(1);
  sendI2CMsg(port, I2CMessage[0], 1);

  while (nI2CStatus[port] == STAT_COMM_PENDING)	wait1Msec(1);
	readI2CReply(port, replyMessage[0], 1);

	return (replyMessage[0]);

}

//---------------------------------------------------------------
static int _PM_ReadInteger (tSensors port, byte i2cAddr, byte pm_register)
{
  ubyte replyMessage[2] ;
  ubyte I2CMessage[4];

  I2CMessage[0] = 2; // Message size
	I2CMessage[1] = i2cAddr; // I2C Device Address
	I2CMessage[2] = pm_register ; // I2C Register to read

	while (nI2CStatus[port] == STAT_COMM_PENDING) wait1Msec(1);
  sendI2CMsg(port, I2CMessage[0], 2);

  while (nI2CStatus[port] == STAT_COMM_PENDING)	wait1Msec(1);
	readI2CReply(port, replyMessage[0], 2);

	return (replyMessage[0] + (replyMessage[1]<<8));

}

//---------------------------------------------------------------
static long _PM_ReadLong (tSensors port, byte i2cAddr, byte pm_register)
{
  ubyte replyMessage[4] ;
  ubyte I2CMessage[4];
  long result ;

  I2CMessage[0] = 2; // Message size
	I2CMessage[1] = i2cAddr; // I2C Device Address
	I2CMessage[2] = pm_register ; // I2C Register to read

	while (nI2CStatus[port] == STAT_COMM_PENDING) wait1Msec(1);
  sendI2CMsg(port, I2CMessage[0], 4);

  while (nI2CStatus[port] == STAT_COMM_PENDING)	wait1Msec(1);
	readI2CReply(port, replyMessage[0], 4);

  result = ((long)replyMessage[3] << 24) +
           ((long)replyMessage[2] << 16) +
           ((long)replyMessage[1] <<  8) +
            (long)replyMessage[0];

  return result ;

}

//---------------------------------------------------------------
static void _PM_ReadString (tSensors port, byte i2cAddr, byte pm_register, string &result)
{
  byte replyMessage[9] ;
  ubyte I2CMessage[4];

  I2CMessage[0] = 2; // Message size
	I2CMessage[1] = i2cAddr; // I2C Device Address
	I2CMessage[2] = pm_register ; // I2C Register to read

	while (nI2CStatus[port] == STAT_COMM_PENDING) wait1Msec(1);
  sendI2CMsg(port, I2CMessage[0], 8);

  while (nI2CStatus[port] == STAT_COMM_PENDING)	wait1Msec(1);
	readI2CReply(port, replyMessage[0], 8);
	replyMessage[8] = 0 ;

	StringFromChars (result, replyMessage) ;
}

#endif
////////////////////////////////////////////////////////////////////////////////
// THAT'S ALL, FOLKS !
////////////////////////////////////////////////////////////////////////////////