///////////////////////////////////////////////////////////////////////////////////////////////////
// Simple testprogram to show the functionality of the Mindsensors PowerMeter interface in RobotC
// The interface/driver is based on the NXC interface written by Deepak Patil for easy migration
//
// Written by Rudolf Lapie
// 17 DEC 2009
///////////////////////////////////////////////////////////////////////////////////////////////////

// The driver routines for the Mindsensors PowerMeter
#include "PM_lib.h"

// CONSTANTS AND GLOBAL VARS
#define SHOW_VOLT     1
#define SHOW_AMP      2
#define SHOW_RESISTOR 3
int display_mode = SHOW_VOLT ;
bool enter_pressed = true ;


// This task is simply to change the effect of the buttons
// Left & Right arrow change the value shown in big
// Orange button resets all internal counters
// Exit button stops the program
task ButtonTask()
{
  nSchedulePriority	 = kHighPriority;

  switch (nNxtButtonPressed)
  {
  	case kLeftButton:
  	    display_mode --;
  	    break ;
		case kRightButton:
		    display_mode ++ ;
		    break ;
		case kEnterButton:
		    enter_pressed = true ;
		    break;
		case kExitButton:
		  StopAllTasks();
		  break;
	}
  if (display_mode > SHOW_RESISTOR) display_mode = SHOW_VOLT ;
  if (display_mode < SHOW_VOLT) display_mode = SHOW_RESISTOR ;
  PlaySoundFile ("! Click.rso") ;
	return;
}

// MAIN TASK : endless loop displaying the values of the PowerMeter
task main ()
{
  string firmware, vendorid, deviceid ;
  int pm_volt, pm_amp ;

	// Change the button handler to our own task
	nNxtButtonTask  = ButtonTask;

	// Configure port S1 to be a low speed I2C sensor.
  SetSensorType(S1, sensorLowSpeed);

  // Read Sensor identification first
  PM_GetFirmware (S1, PM_ADDRESS, firmware) ;
  PM_GetVendorID (S1, PM_ADDRESS, vendorid) ;
  PM_GetDeviceID (S1, PM_ADDRESS, deviceid) ;

  // Endless loop
  while (true) {

    // Read voltage & current from the Mindsensor PowerMeter
    pm_volt = PM_ReadPresentVoltage(S1, PM_ADDRESS) ;
    pm_amp = PM_ReadPresentCurrent(S1, PM_ADDRESS) ;

    eraseDisplay() ;
    nxtDisplayCenteredTextLine (0, "%s - %s", deviceid, firmware) ;
    nxtDisplayCenteredTextLine (1, "- %s - ", vendorid) ;

    // If the orange enter button is pressed, reset all internal counters of the sensor
    if (enter_pressed)
    {
        PM_ResetCounters (S1, PM_ADDRESS) ;
        enter_pressed = false ;

    }

    // Decide which value to show as big and which as small
    switch (display_mode)
    {
       case SHOW_VOLT :
         if (pm_volt < 1000) nxtDisplayCenteredBigTextLine(3, "U:%dmV", pm_volt);
                        else nxtDisplayCenteredBigTextLine(3, "U:%0.2fV", ((float)pm_volt)/1000.0);
          if (pm_amp == 0) nxtDisplayCenteredTextLine(5, "C:%dmA", pm_amp);
                      else nxtDisplayCenteredTextLine(5, "C:%dmA R:%dohm", pm_amp, pm_volt/pm_amp);
          break ;

       case SHOW_AMP :
          if (pm_amp < 1000) nxtDisplayCenteredBigTextLine(3, "C:%dmA", pm_amp);
                        else nxtDisplayCenteredBigTextLine(3, "C:%0.2fA", ((float)pm_amp)/1000.0);
          if (pm_amp == 0) nxtDisplayCenteredTextLine(5, "U:%dmV", pm_volt);
                      else nxtDisplayCenteredTextLine(5, "U:%dmV R:%dohm", pm_volt, pm_volt/pm_amp);
          break ;

       case SHOW_RESISTOR :
          if (pm_amp == 0) nxtDisplayCenteredBigTextLine(3, "R: ???");
                      else nxtDisplayCenteredBigTextLine(3, "R:%dohm", pm_volt/pm_amp);
          nxtDisplayCenteredTextLine(5, "U:%dmV C:%dmA", pm_volt, pm_amp);
          break ;

       default :
         nxtDisplayCenteredBigTextLine (3, "! ERROR !");
         break ;
   }

   // Show the elapsed time
    nxtDisplayCenteredTextLine(7, "time = %0.3fs", ((float)PM_ReadElapsedTime(S1, PM_ADDRESS))/1000.0 ) ;

    wait1Msec(100);
  }
}

///////////////////////////////////////////////////////////////////////////////////////////////////
// THAT's ALL, FOLKS !
///////////////////////////////////////////////////////////////////////////////////////////////////